@@ -87,11 +87,11 @@
 #'
 #' @export
 #'
-LearnerXgboost <- R6::R6Class( # nolint
+LearnerXgboost <- R6::R6Class(
+  # nolint
   classname = "LearnerXgboost",
   inherit = mlexperiments::MLLearnerBase,
   public = list(
-
     #' @description
     #' Create a new `LearnerXgboost` object.
     #'
@@ -104,7 +104,8 @@ LearnerXgboost <- R6::R6Class( # nolint
     #' @examples
     #' LearnerXgboost$new(metric_optimization_higher_better = FALSE)
     #'
-    initialize = function(metric_optimization_higher_better) { # nolint
+    initialize = function(metric_optimization_higher_better) {
+      # nolint
       if (!requireNamespace("xgboost", quietly = TRUE)) {
         stop(
           paste0(
@@ -113,9 +114,22 @@ LearnerXgboost <- R6::R6Class( # nolint
           ),
           call. = FALSE
         )
+
+        if (
+          as.integer(options("mlexperiments.optim.xgb.nrounds")) <
+            as.integer(
+              options("mlexperiments.optim.xgb.early_stopping_rounds")
+            )
+        ) {
+          stop(paste0(
+            "Value of option 'mlexperiments.optim.xgb.nrounds' must be ",
+            "greater than 'mlexperiments.optim.xgb.early_stopping_rounds'"
+          ))
+        }
       }
-      super$initialize(metric_optimization_higher_better =
-                         metric_optimization_higher_better)
+      super$initialize(
+        metric_optimization_higher_better = metric_optimization_higher_better
+      )
       self$environment <- "mllrnrs"
       self$cluster_export <- xgboost_ce()
       private$fun_optim_cv <- xgboost_optimization
@@ -128,15 +142,21 @@ LearnerXgboost <- R6::R6Class( # nolint
 
 
 xgboost_ce <- function() {
-  c("xgboost_optimization", "xgboost_fit",
-    "setup_xgb_dataset", "xgboost_dataset_wrapper")
+  c(
+    "xgboost_optimization",
+    "xgboost_fit",
+    "setup_xgb_dataset",
+    "xgboost_dataset_wrapper",
+    "xgboost_fix_params"
+  )
 }
 
-xgboost_bsF <- function(...) { # nolint
+xgboost_bsF <- function(...) {
+  # nolint
 
   params <- list(...)
 
-  set.seed(seed)#, kind = "L'Ecuyer-CMRG")
+  set.seed(seed) #, kind = "L'Ecuyer-CMRG")
   bayes_opt_xgboost <- xgboost_optimization(
     x = x,
     y = y,
@@ -154,15 +174,27 @@ xgboost_bsF <- function(...) { # nolint
   return(ret)
 }
 
+xgboost_fix_params <- function(params) {
+  sapply(
+    X = names(params),
+    FUN = function(x) {
+      val <- params[[x]]
+      ifelse(is.factor(val), as.character(val), val)
+    },
+    simplify = FALSE,
+    USE.NAMES = TRUE
+  )
+}
+
 # tune lambda
 xgboost_optimization <- function(
-    x,
-    y,
-    params,
-    fold_list,
-    ncores,
-    seed
-  ) {
+  x,
+  y,
+  params,
+  fold_list,
+  ncores,
+  seed
+) {
   stopifnot(
     is.list(params),
     "objective" %in% names(params)
@@ -188,10 +220,17 @@ xgboost_optimization <- function(
     dataset_nrows = nrow(x)
   )
 
+  if ("nrounds" %in% names(params)) {
+    use_nrounds <- params$nrounds
+    params[["nrounds"]] <- NULL
+  } else {
+    use_nrounds <- as.integer(options("mlexperiments.optim.xgb.nrounds"))
+  }
+
   fit_args <- list(
-    params = params,
+    params = xgboost_fix_params(params),
     data = dtrain,
-    nrounds = as.integer(options("mlexperiments.optim.xgb.nrounds")),
+    nrounds = use_nrounds,
     folds = xgb_fids,
     print_every_n = as.integer(options("mlexperiments.xgb.print_every_n")),
     early_stopping_rounds = as.integer(
@@ -215,12 +254,11 @@ xgboost_optimization <- function(
 
   res <- list(
     "metric_optim_mean" = cvfit$evaluation_log[
-      get("iter") == cvfit$best_iteration,
+      get("iter") == cvfit$early_stop$best_iteration,
       get(metric_col)
     ],
-    "nrounds" = cvfit$best_iteration
+    "nrounds" = cvfit$early_stop$best_iteration
   )
-
   return(res)
 }
 
@@ -233,8 +271,10 @@ xgboost_dataset_wrapper <- function(x, y, params) {
   )
   if ("case_weights" %in% names(params)) {
     stopifnot(
-      "late fail: `case_weights` must be of same length as `y`" =
-        length(params$case_weights) == length(y)
+      "late fail: `case_weights` must be of same length as `y`" = length(
+        params$case_weights
+      ) ==
+        length(y)
     )
     dataset_args <- c(
       dataset_args,
@@ -270,11 +310,11 @@ xgboost_fit <- function(x, y, nrounds, ncores, seed, ...) {
   # train final model with best nrounds
   fit_args <- list(
     data = dtrain_full,
-    params = params,
+    params = xgboost_fix_params(params),
     print_every_n = as.integer(options("mlexperiments.xgb.print_every_n")),
     nrounds = nrounds,
-    watchlist = list(
-      train = dtrain_full  # setup a watchlist (the training data here)
+    evals = list(
+      train = dtrain_full # setup a watchlist (the training data here)
     ),
     verbose = as.logical(options("mlexperiments.xgb.verbose"))
   )
@@ -292,24 +332,26 @@ setup_xgb_dataset <- function(x, y, objective, ...) {
     return(setup_surv_xgb_dataset(x, y, objective))
   } else {
     stopifnot(is.atomic(y))
-    # create a xgb.DMatrix
-    dtrain <- xgboost::xgb.DMatrix(x)
-    label <- y
-    xgboost::setinfo(dtrain, "label", label)
     if ("case_weights" %in% names(kwargs)) {
-      xgboost::setinfo(dtrain, "weight", kwargs$case_weights)
+      kwargs$weight <- kwargs$case_weights
+      kwargs[["case_weights"]] <- NULL
     }
-    return(dtrain)
+
+    args <- kdry::list.append(
+      list(
+        data = x,
+        label = y
+      ),
+      kwargs
+    )
+    # create a xgb.DMatrix
+    return(do.call(xgboost::xgb.DMatrix, args))
   }
 }
 
 
 # wrapper function for creating the input data for xgboost
 setup_surv_xgb_dataset <- function(x, y, objective) {
-
-  # create a xgb.DMatrix
-  dtrain <- xgboost::xgb.DMatrix(x)
-
   # for aft-models, the label must be formatted as follows:
   if (objective == "survival:aft") {
     y_lower_bound <- y[, 1]
@@ -318,23 +360,34 @@ setup_surv_xgb_dataset <- function(x, y, objective) {
       y[, 1],
       Inf
     )
-    xgboost::setinfo(dtrain, "label_lower_bound", y_lower_bound)
-    xgboost::setinfo(dtrain, "label_upper_bound", y_upper_bound)
+    args <- list(
+      data = x,
+      label_lower_bound = y_lower_bound,
+      label_upper_bound = y_upper_bound
+    )
   } else if (objective == "survival:cox") {
     # Cox regression for right censored survival time data (negative values
     # are considered right censored). Note that predictions are returned on
     # the hazard ratio scale (i.e., as HR = exp(marginal_prediction) in
     # the proportional hazard function h(t) = h0(t) * HR).
     label <- ifelse(y[, 2] == 1, y[, 1], -y[, 1])
-    xgboost::setinfo(dtrain, "label", label)
+    args <- list(
+      data = x,
+      label = label
+    )
   }
-
-  return(dtrain)
+  return(do.call(xgboost::xgb.DMatrix, args))
 }
 
 
 xgboost_predict <- function(model, newdata, ncores, ...) {
   kwargs <- list(...)
+  if ("reshape" %in% names(kwargs)) {
+    reshape <- kwargs$reshape
+    kwargs[["reshape"]] <- NULL
+  } else {
+    reshape <- NA
+  }
   args <- kdry::list.append(
     list(
       object = model,
@@ -343,8 +396,8 @@ xgboost_predict <- function(model, newdata, ncores, ...) {
     kwargs
   )
   preds <- do.call(stats::predict, args)
-  if (!is.null(kwargs$reshape)) {
-    if (isTRUE(kwargs$reshape)) {
+  if (!is.na(reshape)) {
+    if (isTRUE(reshape)) {
       preds <- kdry::mlh_reshape(preds)
     }
   }
